CHAINS_URL = "chains"
IDENTITIES_URL = "identities"
KEYS_STRING = "keys"
SEARCH_URL = "search"
IDENTITY_URL = "identities"
ENTRIES_URL = "entries"
FIRST_URL = "first"
LAST_URL = "last"
RECEIPTS_URL = "receipts"
ANCHORS_URL = "anchors"
PRIVATE_PREFIX_BYTES = bytearray([0x03, 0x45, 0xf3, 0xd0, 0xd6])
PUBLIC_PREFIX_BYTES = bytearray([0x03, 0x45, 0xef, 0x9d, 0xe0])
BASE64_ENCODE = "base64"
